@@ -13,6 +13,8 @@ import java.net.*;
 
 public class AAAARecord extends Record {
 
+private static final long serialVersionUID = -4588601512069748050L;
+
 private InetAddress address;
 
 AAAARecord() {}
@@ -36,7 +38,11 @@ AAAARecord(Name name, int dclass, long ttl, InetAddress address) {
 
 void
 rrFromWire(DNSInput in) throws IOException {
-	address = InetAddress.getByAddress(in.readByteArray(16));
+	if (name == null)
+		address = InetAddress.getByAddress(in.readByteArray(16));
+	else
+		address = InetAddress.getByAddress(name.toString(),
+						   in.readByteArray(16));
 }
 
 void
