@@ -12,7 +12,9 @@ import java.text.*;
  * @author Brian Wellington
  */
 
-public class Name implements Comparable {
+public class Name implements Comparable, Serializable {
+
+private static final long serialVersionUID = -7257019940971525644L;
 
 private static final int LABEL_NORMAL = 0;
 private static final int LABEL_COMPRESSION = 0xC0;
@@ -80,26 +82,6 @@ private
 Name() {
 }
 
-private final void
-dump(String prefix) {
-	String s;
-	try {
-		s = toString();
-	} catch (Exception e) {
-		s = "<unprintable>";
-	}
-	System.out.println(prefix + ": " + s);
-
-	int labels = labels();
-	for (int i = 0; i < labels; i++)
-		System.out.print(offset(i) + " ");
-	System.out.println("");
-
-	for (int i = 0; name != null && i < name.length; i++)
-		System.out.print((name[i] & 0xFF) + " ");
-	System.out.println("");
-}
-
 private final void
 setoffset(int n, int offset) {
 	if (n >= MAXOFFSETS)
@@ -360,8 +342,7 @@ fromConstantString(String s) {
  */
 public
 Name(DNSInput in) throws WireParseException {
-	int len, pos, currentpos;
-	Name name2;
+	int len, pos;
 	boolean done = false;
 	byte [] label = new byte[MAXLABEL + 1];
 	boolean savedState = false;
@@ -492,6 +473,30 @@ wild(int n) {
 	}
 }
 
+/**
+ * Returns a canonicalized version of the Name (all lowercase).  This may be
+ * the same name, if the input Name is already canonical.
+ */
+public Name
+canonicalize() {
+	boolean canonical = true;
+	for (int i = 0; i < name.length; i++) {
+		if (lowercase[name[i] & 0xFF] != name[i]) {
+			canonical = false;
+			break;
+		}
+	}
+	if (canonical)
+		return this;
+
+	Name newname = new Name();
+	newname.appendSafe(name, offset(0), getlabels());
+	for (int i = 0; i < newname.name.length; i++)
+		newname.name[i] = lowercase[newname.name[i] & 0xFF];
+
+	return newname;
+}
+
 /**
  * Generates a new Name to be used when following a DNAME.
  * @param dname The DNAME record to follow.
@@ -604,10 +609,11 @@ byteString(byte [] array, int pos) {
 
 /**
  * Convert a Name to a String
+ * @param omitFinalDot If true, and the name is absolute, omit the final dot.
  * @return The representation of this name as a (printable) String.
  */
 public String
-toString() {
+toString(boolean omitFinalDot) {
 	int labels = labels();
 	if (labels == 0)
 		return "@";
@@ -618,17 +624,28 @@ toString() {
 		int len = name[pos];
 		if (len > MAXLABEL)
 			throw new IllegalStateException("invalid label");
-		if (len == 0)
+		if (len == 0) {
+			if (!omitFinalDot)
+				sb.append('.');
 			break;
+		}
+		if (i > 0)
+			sb.append('.');
 		sb.append(byteString(name, pos));
-		sb.append('.');
 		pos += (1 + len);
 	}
-	if (!isAbsolute())
-		sb.deleteCharAt(sb.length() - 1);
 	return sb.toString();
 }
 
+/**
+ * Convert a Name to a String
+ * @return The representation of this name as a (printable) String.
+ */
+public String
+toString() {
+	return toString(false);
+}
+
 /**
  * Retrieve the nth label of a Name.  This makes a copy of the label; changing
  * this does not change the Name.
